#!/usr/bin/env python

#import modules
import random
import pygtk
pygtk.require('2.0')
import gtk

#define application class
class DrawingThing():
    def __init__(self):
        window = gtk.Window()
        window.set_title("Drawing Thing")
        vbox = gtk.VBox(False, 0)
        window.add(vbox)
        vbox.show()
        window.connect("destroy", lambda w: gtk.main_quit())
        #create a drawing area
        self.drawing_area = gtk.DrawingArea()
        self.drawing_area.set_size_request(200, 200)
        vbox.pack_start(self.drawing_area, True, True, 0)
        self.drawing_area.show()
    
        # attach relevant signals
        self.drawing_area.connect("button_press_event", self.button_press_event)
        self.drawing_area.connect("expose_event", self.expose_event)
        #self.drawing_area.set_events(gtk.gdk.EXPOSURE_MASK)
        
        #set up the drawing area to respond to particular events
        self.drawing_area.set_events(gtk.gdk.EXPOSURE_MASK
                                | gtk.gdk.LEAVE_NOTIFY_MASK
                                | gtk.gdk.BUTTON_PRESS_MASK
                                | gtk.gdk.POINTER_MOTION_MASK
                                | gtk.gdk.POINTER_MOTION_HINT_MASK)
                                
        #set the drawing mode
        self.drawmode=0
        #add some buttons to activate drawing modes
        for i in range(4):
            button = gtk.Button("Draw Type "+str(i))
            vbox.pack_start(button, False, False, 0)
            button.connect_object("clicked", self.draw_type, i)
            button.show()
            
        #Show the window
        window.show()
        self.draw_type(0)
    
    def button_press_event(self, widget, event):
        #button is pressed, draw a rectangle on the screen
        x= event.x
        y= event.y
        cr = self.drawing_area.window.cairo_create()
        cr.set_source_rgb(0, 0, 1)
        cr.rectangle(x, y, 2, 2)
        cr.fill()
        return False

    def expose_event(self, widget, event):
        #an exposure event has occured, redraw the screen
        x , y, width, height = event.area
        cr = self.drawing_area.window.cairo_create()
        cr.rectangle(x,y,width,height)
        cr.clip()
        cr.set_source_rgb(1, 0.8, 0.8)
        cr.rectangle(0, 0, width, height)
        cr.fill()
        #once the screen is clear, initiate the drawing mode
        self.draw_type(self.drawmode)
        return False

    def draw_type(self, type):
        # handles the drawing modes 
        x, y, width, height =self.drawing_area.get_allocation()
        if (type ==0):
            # Default mode, do nothing (also clears screen)
            x, y, w, h=self.drawing_area.get_allocation()
            cr = self.drawing_area.window.cairo_create()
            cr.set_source_rgb(1, 0.8, 0.8)
            cr.rectangle(0, 0, width, height)
            cr.fill()
            return True
        if (type == 1):
            # Draw random rectangles
            x, y, w, h=self.drawing_area.get_allocation()
            cr = self.drawing_area.window.cairo_create()
            rr=random.randrange
            rf=random.random
            for i in range (25):
                x=rr(w-90)
                y=rr(h-90)
                cr.set_source_rgba(rf(), rf(), rf(), 0.3)
                cr.rectangle(x, y, 90, 90)
                cr.fill()
            return True
                
        if (type == 2):   
            # draw Henon Map
            x, y, w, h=self.drawing_area.get_allocation()
            a=0.2
            b=0.9991
            xmax = ymax = 6
            xmin = ymin = -6
            x1=y1=0
            xfactor=w/(xmax-xmin)
            yfactor=h/(ymax-ymin)
            for i in range(7000):
                cr = self.drawing_area.window.cairo_create()
                x2=1-a*x1*x1+y1
                y2=b*x1
                if ((x1<xmin) | (x1>xmax) | (y1<ymin) | (y1>ymax)):
                    return #point has escaped
                
                cr.set_source_rgb(0, 0, 0)
                xx=(x1-xmin)*xfactor
                yy=(y1-ymin)*yfactor
                cr.rectangle(xx, yy, 1, 1)
                cr.stroke()
                x1=x2
                y1=y2
        #add your own drawing modes here!
        return True
        
        
def main():
    gtk.main()
    

if __name__ == "__main__":
    DrawingThing()
    main()
